<?php

namespace App\Http\Controllers\Backend\Server\System;

use App\Exceptions\ApiException;
use App\Http\Controllers\Controller;
use App\Http\Services\Backend\Server\System\PermissionService;
use App\Http\Services\Backend\Server\System\RoleService;
use App\Models\System\Menu;
use App\Models\System\Role;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

class PermissionController extends Controller
{
    private PermissionService $permissionService;

    public function __construct(PermissionService $permissionService)
    {
        $this->permissionService = $permissionService;
    }

    public function index()
    {
        // 处理首页逻辑
        return 'index';
    }

    public function show($id)
    {
//        $result = $this->permissionService->getPermissions();
        // 处理显示单个用户的逻辑
        return self::success();
    }

    public function create()
    {
        // 显示创建用户的表单
    }

    public function store(Request $request)
    {
        // 创建分配逻辑
    }

    public function edit($id)
    {
        // 显示编辑用户的表单
    }

    /**
     * @throws ApiException
     */
    public function update(Request $request, $id): JsonResponse
    {
        // 更新用户逻辑
        $params = $request->all();
        $this->permissionService->assignPermission($params, $id);
        return self::success(true);
    }

    public function destroy($id)
    {
        // 删除用户逻辑
    }

    public function getRoleMenus(int $id): JsonResponse
    {
        $res = $this->permissionService->getPermissions($id, Role::class);
        return self::success($res);
    }

    public function assignRoleMenu(Request $request, $id)
    {
        $params = ['permission' => $request->post('menuIds', [])];
        $this->permissionService->assignRolePermission($params, $id);
        return self::success(true);
    }

    // 用户管理-分配角色-获取用户角色
    public function getUserRoles(int $id): JsonResponse
    {
        $res = $this->permissionService->getUserRoles($id);
        return self::success($res);
    }

    // 用户管理-分配用户角色
    public function assignUserRole(Request $request)
    {
        $data = $request->only(['userId', 'roleIds']);
        $this->permissionService->assignUserRole($data);
        return self::success(true);
    }
}
