<?php

namespace App\Http\Controllers\Backend\Server\System;

use App\Http\Controllers\Controller;
use App\Http\Requests\Backend\Server\System\UserRequest;
use App\Http\Services\Backend\Server\System\UserService;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

class UserController extends Controller
{
    private UserService $userService;

    public function __construct(UserService $userService)
    {
        $this->userService = $userService;
    }

    public function index(Request $request): JsonResponse
    {
        $params = $request->all();
        $result = $this->userService->getUserList($params);
        return self::success($result);
    }

    public function show(int $id)
    {
        // 处理显示单个用户的逻辑
        $result = $this->userService->getUser($id);
        return self::success($result);
    }

    public function create()
    {
        // 显示创建用户的表单
    }

    public function store(UserRequest $request)
    {
        // 存储新创建的用户逻辑
        $data = $request->safe(['username', 'password', 'nickname', 'deptId', 'mobile', 'email', 'sex', 'postIds', 'remark', 'status']);
        $result = $this->userService->createUser($data);
        return self::success($result);
//        User::query()->create([
//            'name' => $request->string('name'),
//            'password' => Hash::make($request->string('password')),
//        ]);
//
//        return response()->noContent();

//        event(new Registered($user));
//
//        Auth::login($user);
//
//        return response()->noContent();
    }

    public function edit($id)
    {
        // 显示编辑用户的表单
    }

    public function update(Request $request, int $id)
    {
        // 更新用户逻辑
        $data = $request->all();
        $this->userService->updateUser($data, $id);
        return self::success(true);
    }

    public function destroy($id)
    {
        // 删除用户逻辑
    }

    public function status(Request $request): JsonResponse
    {
        $this->userService->setStatus($request->only(['id', 'status']));
        return self::success(true);
    }
}
