<?php
/**
 * @Name
 * @Description
 * @Author 刘学玺
 * @Date 2024/9/26 11:58
 */

namespace App\Http\Controllers\Frontend\Client\Auth;

use App\Exceptions\ApiException;
use App\Http\Controllers\Controller;
use App\Http\Requests\Request;
use App\Http\Services\Frontend\Client\Auth\AuthenticatedService;
use Illuminate\Http\JsonResponse;

class AuthenticatedController extends Controller
{
    protected AuthenticatedService $authService;

    public function __construct(AuthenticatedService $authService)
    {
        $this->authService = $authService;
    }

    /**
     * @throws ApiException
     */
    public function store(Request $request): JsonResponse
    {
        $request->validate([
            'mobile' => ['required', 'regex:/^1[345789][0-9]{9}$/'],
            'code' => ['required'],
        ], [
            'mobile.required' => '手机号不能为空！',
            'mobile.regex' => '手机号码格式不正确！',
            'code.required' => '手机验证码不能为空！',
        ]);
        $params = $request->only(['mobile','code']);
        return self::success($this->authService->login($params));
    }

    /**
     * Method : 发送验证码
     * @param Request $request
     * @return JsonResponse
     * @throws ApiException
     */
    public function send(Request $request): JsonResponse
    {
        $params = $request->only(['mobile']);
        $this->authService->send($params);
        return self::success(true);
    }

}
