<?php

namespace App\Http\Controllers\Frontend\Client\Coach;

use App\Exceptions\ApiException;
use App\Http\Controllers\Controller;
use App\Http\Requests\Frontend\Client\Coach\UserRequest;
use App\Http\Services\Frontend\Client\Coach\UserService;
use Illuminate\Http\JsonResponse;

class UserController extends Controller
{
    protected UserService $service;

    public function __construct(UserService $service)
    {
        $this->service = $service;
    }

    /**
     * Notes :
     * Method : 技师列表
     * @param UserRequest $request
     * @return JsonResponse
     */
    public function index(UserRequest $request): JsonResponse
    {
        // 处理首页逻辑
        $result = $this->service->getUserPage($request->safe()->toArray());
        return self::success($result);
    }

    /**
     * Notes :
     * Method : 技师详情
     * @param UserRequest $request
     * @param int $id
     * @return JsonResponse
     */
    public function show(UserRequest $request, int $id): JsonResponse
    {
        $result = $this->service->getUser($request->safe()->toArray(), $id);
        return self::success($result);
    }

    /**
     * Notes :
     * Method : 技师申请
     * @param UserRequest $request
     * @return JsonResponse
     * @throws ApiException
     */
    public function apply(UserRequest $request): JsonResponse
    {
        $this->service->apply($request->safe()->toArray());
        return self::success(true);
    }
}
