<?php
/**
 * @Name
 * @Description
 * @Author 刘学玺
 * @Date 2024/9/26 11:58
 */

namespace App\Http\Controllers\Frontend\Client\Member;

use App\Exceptions\ApiException;
use App\Http\Controllers\Controller;
use App\Http\Requests\Frontend\Client\Member\AddressRequest;
use App\Http\Requests\Frontend\Client\Member\UserRequest;
use App\Http\Requests\Request;
use App\Http\Services\Frontend\Client\Auth\AuthenticatedService;
use App\Http\Services\Frontend\Client\Member\AddressService;
use App\Http\Services\Frontend\Client\Member\UserService;
use Illuminate\Http\JsonResponse;

class AddressController extends Controller
{
    protected AddressService $service;

    public function __construct(AddressService $service)
    {
        $this->service = $service;
    }

    /**
     * Notes :
     * Method : 用户地址列表
     * @return JsonResponse
     */
    public function index(): JsonResponse
    {
        return self::success($this->service->getAddressList());
    }

    /**
     * @throws ApiException
     */
    public function store(AddressRequest $request): JsonResponse
    {
        return self::success($this->service->createAddress($request->safe()->toArray()));
    }

    /**
     * Notes :
     * Method : 用户地址详情
     * @param $address_id
     * @return JsonResponse
     */
    public function show($address_id): JsonResponse
    {
        return self::success($this->service->getAddress($address_id));
    }

    /**
     * Notes :
     * Method : 更新用户信息
     * @param AddressRequest $request
     * @param $address_id
     * @return JsonResponse
     * @throws ApiException
     */
    public function update(AddressRequest $request, $address_id): JsonResponse
    {
        $params = $request->safe()->toArray();
        $this->service->updateAddress($params, $address_id);
        return self::success(true);
    }

    /**
     * Notes :
     * Method : 注销用户
     */
    public function destroy(int $id): JsonResponse
    {
        $this->service->delAddress($id);
        return self::success(true);
    }

    public function default(): JsonResponse
    {
        return self::success($this->service->getDefault());
    }

    public function setDefault(int $id): JsonResponse
    {
        $this->service->setDefault($id);
        return self::success(true);
    }

}
