<?php
/**
 * @Name
 * @Description
 * @Author 刘学玺
 * @Date 2024/9/26 11:58
 */

namespace App\Http\Controllers\Frontend\Client\Member;

use App\Exceptions\ApiException;
use App\Http\Controllers\Controller;
use App\Http\Requests\Frontend\Client\Member\UserRequest;
use App\Http\Requests\Request;
use App\Http\Services\Frontend\Client\Auth\AuthenticatedService;
use App\Http\Services\Frontend\Client\Member\UserService;
use Illuminate\Http\JsonResponse;

class UserController extends Controller
{
    protected UserService $service;

    public function __construct(UserService $service)
    {
        $this->service = $service;
    }

    /**
     * Notes :
     * Method : 用户信息
     * @return JsonResponse
     */
    public function show(): JsonResponse
    {
        return self::success($this->service->getUser());
    }

    /**
     * Notes :
     * Method : 更新用户信息
     * @param UserRequest $request
     * @return JsonResponse
     */
    public function update(UserRequest $request): JsonResponse
    {
        $params = $request->all();
        $this->service->updateUser($params);
        return self::success(true);
    }

    /**
     * Notes :
     * Method : 注销用户
     */
    public function destroy(Request $request): JsonResponse
    {
        $this->service->delUser($request);
        return self::success(true);
    }

    public function address(): JsonResponse
    {
        return self::success($this->service->address());
    }

    public function qrcode(): \Illuminate\Foundation\Application|\Illuminate\Http\Response|\Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\Routing\ResponseFactory
    {
        $result = $this->service->qrcode();
        return response($result->getString())->header('Content-Type', 'image/png');
    }

    public function balance(): JsonResponse
    {
        $result = $this->service->getBalancePage();
        return self::success($result);
    }
}
