<?php
/**
 * @Name
 * @Description
 * @Author 刘学玺
 * @Date 2024/9/23 10:06
 */

namespace App\Http\Controllers\Frontend\Client\Service;

use App\Exceptions\ApiException;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Frontend\Client\Order\Config;
use App\Http\Requests\Frontend\Client\Service\OrderRequest;
use App\Http\Requests\Request;
use App\Http\Services\Frontend\Client\Service\OrderService;
use App\Models\Coach\User;
use App\Models\Service\Order;
use Endroid\QrCode\Builder\Builder;
use Endroid\QrCode\Writer\PngWriter;
use Illuminate\Foundation\Application;
use Illuminate\Http\JsonResponse;

//use SimpleSoftwareIO\QrCode\Facades\QrCode;

class OrderController extends Controller
{

    protected OrderService $orderService;

    public function __construct(OrderService $orderService)
    {
        $this->orderService = $orderService;
    }

    public function index(Request $request): JsonResponse
    {
        $params = $request->all();
        $res = $this->orderService->getOrderPage($params);
        return self::success($res);
    }

    /**
     * @throws ApiException
     */
    public function store(OrderRequest $request): JsonResponse
    {
        $params = $request->safe()->toArray();
        $res = $this->orderService->createOrder($params);
        return self::success($res);
    }

    public function show(Request $request, int $id): JsonResponse
    {
        return self::success($this->orderService->getOrder($id));
    }

    /**
     * @throws ApiException
     */
    public function update(Request $request, int $id): JsonResponse
    {
        $params = $request->all();
        $this->orderService->updateOrder($params, $id);
        return self::success(true);
    }

    public function destroy(int $id): JsonResponse
    {
        $this->orderService->delOrder($id);
        return self::success(true);
    }

    public function confirm(Request $request): JsonResponse
    {
        $params = $request->all();
        $res = $this->orderService->confirmOrder($params);
        return self::success($res);
    }

    /**
     * @throws ApiException
     */
    public function compute(Request $request): JsonResponse
    {
        $params = $request->all();
        $res = $this->orderService->computePrice($params);
        return self::success($res);
    }

    public function start()
    {

    }

    /**
     * Notes :
     * Method : 结束订单
     * @throws ApiException
     */
    public function end(Request $request): JsonResponse
    {
        $params = $request->all();
        $this->orderService->updateOrder([...$params, 'status' => 6]);
        return self::success(true);
    }

    /**
     * @throws ApiException
     */
    public function cancel(int $id): JsonResponse
    {
        $this->orderService->cancelOrder($id);
        return self::success(true);
    }

    public function qrCode(Request $request)
    {
        $params = $request->all();
        $result = $this->orderService->qrCode($params);
        return response($result->getString())->header('Content-Type', 'image/png');
    }

    public function grabList(int $id)
    {
        $result = $this->orderService->getGrabList($id);
        return self::success($result);
    }

    /**
     * @throws ApiException
     */
    public function setGrab(Request $request, int $id): JsonResponse
    {
        $coach_id = $request->integer('coachId', 0);
        $this->orderService->setGrab($id, $coach_id);
        return self::success(true);
    }

}




