<?php

use App\Http\Controllers\Backend\Server\System\UserController;
use App\Http\Controllers\Frontend\Client\CoachController as ClientCoachController;
use App\Http\Controllers\Frontend\Client\Coach\UserController as ClientCoachUserController;
use App\Http\Controllers\Frontend\Server\Coach\UserController as ServerCoachUserController;
use App\Http\Controllers\Frontend\Client\Service\ProjectController as ClientProjectController;
use App\Http\Controllers\Frontend\Server\Service\ProjectController as ServerProjectController;
use App\Http\Controllers\Frontend\Client\Service\OrderController as ClientServiceOrderController;
use App\Http\Controllers\Frontend\Client\Auth\AuthenticatedController as ClientAccountAuthController;
use App\Http\Controllers\Frontend\Client\Service\CategoryController as ClientServiceCategory;
use App\Http\Controllers\Frontend\Client\Member\UserController as ClientMemberUserController;
use App\Http\Controllers\Frontend\Client\Auth\WechatAuthenticatedController as ClientWechatAuthenticatedController;
use App\Http\Controllers\Frontend\Client\Member\ToolController as ClientMemberToolController;
use App\Http\Controllers\Frontend\Client\Member\AddressController as ClientMemberAddressController;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

Route::prefix('client')->group(function () {
    Route::get('/', function () {
        return ['Laravel' => 'frontend client'];
    });
});


Route::middleware(['auth:sanctum'])->get('/user', function (Request $request) {
    return $request->user();
});

# 客户
Route::prefix('client')->group(function () {
    # 登录
    Route::prefix('login')->group(function () {
        Route::post('/', [ClientAccountAuthController::class, 'store']);
        Route::post('send', [ClientAccountAuthController::class, 'send']);
    });
    # 微信
    Route::prefix('wechat')->group(function () {
        Route::get('/oauth', [ClientWechatAuthenticatedController::class, 'oauth']);
        Route::get('/callback', [ClientWechatAuthenticatedController::class, 'callback']);
        Route::get('/login', [ClientWechatAuthenticatedController::class, 'store']);
        Route::get('/bind', [ClientWechatAuthenticatedController::class, 'bind']);

        Route::get('/signature', [ClientWechatAuthenticatedController::class, 'signature']);
        # 支付
        Route::middleware(['auth:sanctum'])->group(function () {
            Route::get('/payment', [\App\Http\Controllers\Frontend\Client\Wechat\PaymentController::class, 'payment']);
        });
        Route::post('/payment/notify', [\App\Http\Controllers\Frontend\Client\Wechat\PaymentController::class, 'notify']);

    });

    # 首页
    Route::prefix('service')->group(function () {
        Route::get('category', [ClientServiceCategory::class, 'index']);
        Route::get('project', [ClientProjectController::class, 'index']);
        Route::get('project/{id}', [ClientProjectController::class, 'show']);


        Route::middleware(['auth:sanctum'])->group(function () {
            # 订单

            # 工单 - 查询
            Route::get('order/grab/{id}', [ClientServiceOrderController::class, 'grabList']);
            Route::put('order/grab/{id}', [ClientServiceOrderController::class, 'setGrab']);
            Route::get('order/confirm', [ClientServiceOrderController::class, 'confirm']);
            Route::get('order/compute', [ClientServiceOrderController::class, 'compute']);
            Route::put('order/cancel/{id}',[ClientServiceOrderController::class, 'cancel']);
            Route::get('order/qrCode',[ClientServiceOrderController::class, 'qrCode']);
            Route::resource('order', ClientServiceOrderController::class);

        });

    });


    Route::prefix('coach')->group(function () {
        # 申请技师
        Route::post('apply', [ClientCoachUserController::class, 'apply'])->middleware('auth:sanctum');
        Route::get('/', [ClientCoachUserController::class, 'index']);
        Route::get('/{id}', [ClientCoachUserController::class, 'show']);
    });

    Route::prefix('member')->middleware(['auth:sanctum'])->group(function () {
        # 余额明细
        Route::get('balance', [ClientMemberUserController::class, 'balance']);
        # 邀请码
        Route::get('qrcode', [ClientMemberUserController::class, 'qrcode']);
        Route::get('address/default', [ClientMemberAddressController::class, 'default']);
        Route::put('address/default/{id}', [ClientMemberAddressController::class, 'setDefault']);

        Route::resource('address', ClientMemberAddressController::class);
        Route::post('upload', [ClientMemberToolController::class, "upload"]);
        Route::get('/', [ClientMemberUserController::class, 'show']);
        Route::delete('/', [ClientMemberUserController::class, 'destroy']);
        Route::put('/', [ClientMemberUserController::class, 'update']);
    });


});

# 技师端
Route::prefix('server')->group(function () {
    Route::get('/', function () {
        return ['Laravel' => 'frontend server'];
    });


    Route::prefix('coach')->group(function () {
        # 获取技师详情
        Route::get('user', [ServerCoachUserController::class, 'show']);
        Route::put('user', [ServerCoachUserController::class, 'update']);

    });

    Route::prefix('service')->group(function () {
        Route::get('project', [ServerProjectController::class, 'index']);
        Route::post('project', [ServerProjectController::class, 'update']);
    });
});
