<?php

use App\Http\Controllers\Backend\Server\System\AuthController;
use App\Http\Controllers\Backend\Server\System\CaptchaController;
use App\Http\Controllers\Backend\Server\System\DeptController;
use App\Http\Controllers\Backend\Server\System\DictDataController;
use App\Http\Controllers\Backend\Server\System\DictTypeController;
use App\Http\Controllers\Backend\Server\System\MenuController;
use App\Http\Controllers\Backend\Server\System\NotifyMessageController;
use App\Http\Controllers\Backend\Server\System\PermissionController;
use App\Http\Controllers\Backend\Server\System\PostController;
use App\Http\Controllers\Backend\Server\System\RoleController;
use App\Http\Controllers\Backend\Server\System\UserController;
use App\Http\Controllers\Backend\Server\Member\UserController as MemberUserController;
use App\Http\Controllers\Backend\Server\Member\ConfigController as MemberConfigController;
use App\Http\Controllers\Backend\Server\Service\CategoryController as ServiceCategoryController;
use App\Http\Controllers\Backend\Server\Service\ProjectController as ServiceProjectController;
use App\Http\Controllers\Backend\Server\Coach\UserController as CoachUserController;
use App\Http\Controllers\Backend\Server\Coach\ApplyController as CoachApplyController;

use Illuminate\Support\Facades\Route;

Route::get('/', function () {
    return ['Laravel' => app()->version()];
});

Route::prefix('client')->group(function () {
    Route::get('/', function () {
        return ['Laravel' => 'Backend client'];
    });
});

//Route::prefix('server')->group(function () {

# 注册
//    Route::post('/register', [RegisteredUserController::class, 'store'])
//        ->middleware('guest')
//        ->name('register');

# 服务中心
Route::prefix('service')->group(function () {
    # 服务分类
    Route::get('category/simple', [ServiceCategoryController::class, 'simple']);
    Route::resource('category', ServiceCategoryController::class);
    # 服务项目
    Route::resource('project', ServiceProjectController::class);

});
# 会员中心
Route::prefix('member')->group(function () {
    # 会员管理
    Route::resource('user', MemberUserController::class);
    # 会员配置
    Route::get('config/get', [MemberConfigController::class, 'show']);
    Route::put('config/save', [MemberConfigController::class, 'update']);

});

# 技工中心
Route::prefix('coach')->group(function () {
    Route::get('user/apply/{id}', [CoachUserController::class, 'apply']);
    Route::put('user/apply/{id}', [CoachUserController::class, 'doApply']);
    Route::put('user/reapply/{id}', [CoachUserController::class, 'doReApply']);

    Route::get('user/auth/{id}', [CoachUserController::class, 'auth']);
    Route::put('user/auth/{id}', [CoachUserController::class, 'doAuth']);

    Route::resource('user', CoachUserController::class);
});

Route::prefix('system')->group(function () {
    Route::post('auth/login', [AuthController::class, 'login']);
    Route::get('auth/get-permission-info', [AuthController::class, 'getPermissionInfo']);

    Route::get('notify-message/get-unread-count', [NotifyMessageController::class, 'getUnreadCount']);

    // 用户状态
    Route::put('user/update-status', [UserController::class, 'status']);
    // 用户管理
    Route::resource('user', UserController::class);

    Route::get('menu/simple-list', [MenuController::class, 'simpleList']);
    Route::resource('menu', MenuController::class);


    Route::get('dict-data/simple-list', [DictDataController::class, 'simpleList']);
    Route::resource('dict-type', DictTypeController::class);
    Route::resource('dict-data', DictDataController::class);

    Route::get('role/simple-list', [RoleController::class, 'simpleList']);
    Route::resource('role', RoleController::class);
//    Route::get('role/permissions/{id}',[ RoleController::class , 'permissions']);
//    Route::resource('captcha', CaptchaController::class);
//    Route::resource('permissions', PermissionController::class);
    Route::get('permission/role_menus/{id}', [PermissionController::class, 'getRoleMenus']);
    Route::post('permission/assign_role_menu/{id}', [PermissionController::class, 'assignRoleMenu']);

    // 获取用户角色
    Route::get('permission/user-roles/{id}', [PermissionController::class, 'getUserRoles']);
    // 分配用户角色
    Route::post('permission/assign-user-role', [PermissionController::class, 'assignUserRole']);

    // 部门
    Route::get('dept/simple-list', [DeptController::class, 'simple']);
    Route::resource('dept', DeptController::class);

    // 岗位
    Route::get('post/simple-list', [PostController::class, 'simple']);

});
//});

require __DIR__ . '/auth.php';
